# -*- coding: utf-8 -*-

from typing import Union
import urllib3

from indico.config import IndicoConfig
from indico.http.client import HTTPClient
from indico.client.request import HTTPRequest, RequestChain


class IndicoClient:
    """
    The Indico GraphQL Client. 

    IndicoClient is the primary way to interact with the Indico Platform.
    
    Args:
        config= (IndicoConfig, optional): IndicoConfig object with environment configuration
    
    Returns:
        IndicoConfig object
    
    Raises:
        RuntimeError: If api_token_path does not exist.
    """

    def __init__(self, config: IndicoConfig = None):
        if not config:
            config = IndicoConfig()
        if not config.verify_ssl:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self.config = config
        self._http = HTTPClient(config)

    def _handle_request_chain(self, chain: RequestChain):
        response = None
        for request in chain.requests():
            if isinstance(request, HTTPRequest):
                response = self._http.execute_request(request)
                chain.previous = response
            elif isinstance(request, RequestChain):
                response = self._handle_request_chain(request)
                chain.previous = response

        if chain.result:
            return chain.result
        return response

    def call(self, request: Union[HTTPRequest, RequestChain]):
        """
        Make a call to the Indico IPA Platform

        Args:
            request (GraphQLRequest or RequestChain): GraphQL request to send to the Indico Platform
        
        Returns:
            Response appropriate to the class of the provided request parameter. Often JSON but not always.
        
        Raises:
            IndicoRequestError: With errors in processing the request
        """

        if isinstance(request, RequestChain):
            return self._handle_request_chain(request)
        elif request and isinstance(request, HTTPRequest):
            return self._http.execute_request(request)



from indico.types import BaseType, List
from . import SubmissionFile

VALID_SUBMISSION_STATUSES = [
    "COMPLETE",
    "FAILED",
    "PENDING_REVIEW",
    "PROCESSING",
    "PENDING_ADMIN_REVIEW",
    "PENDING_AUTO_REVIEW",
]


SUBMISSION_RESULT_VERSIONS = ["ONE", "TWO", "OLDEST_SUPPORTED", "LATEST"]


class Submission(BaseType):
    f"""
    A Submission in the Indico Platform.

    Submissions represent a single input which has been sent for processing by a
    specific workflow. A submission consists of SubmissionFiles, generally a PDF.
    Processing broadly consists of an input processor, a series of processors and
    components associated with particular docbots, and finally an output processor
    to generate the result file.

    Attributes:
        id (int): the Submission id
        dataset_id (int): the Dataset id
        workflow_id (int): the Workflow id
        status (str): status of the submission. One of
            {VALID_SUBMISSION_STATUSES}
        input_files (list[SubmissionFile]): the SubmissionFiles for the Submission
        input_file (str): URL of the first input datafile within the Indico Platform.
        input_filename (str): name of the first original file
        result_file (str): URL of the result datafile within the Indico Platform
        retrieved (bool): Whether the submission has been retrieved by a user
            This flag is set manually by users.
        errors (str): Any errors raised while processing the submission

    """

    id: int
    dataset_id: int
    workflow_id: int
    status: str
    input_files: List[SubmissionFile]
    input_file: str
    input_filename: str
    result_file: str
    retrieved: bool
    errors: str



from indico.types import BaseType


class SubmissionFile(BaseType):
    f"""
    A Submission File in the Indico Platform.

    Submissions files represent a single document, and can be grouped together under
    a single submission to a submission to a workflow.

    Attributes:
        id (int): the Submission file id
        filepath (str): URL of the input datafile within the Indico Platform.
        filename (str): name of the original file
        submission_id (int): the parent Submission id

    """

    id: int
    filepath: str
    filename: str
    submission_id: int
